/**
 * @file
 * Entity Reference Tree JavaScript file.
 */

// Codes run both on normal page loads and when data is loaded by AJAX (or BigPipe!)
// @See https://www.drupal.org/docs/8/api/javascript-api/javascript-api-overview
(function ($, Drupal, once) {
  Drupal.behaviors.entityReferenceTreeInline = {
    attach: function (context, settings) {
      const entityJSTreeInline = once('jstreeBehavior', '.inline-entity-reference-tree-wrapper', context);
      entityJSTreeInline.forEach(function (treeContainerElement) {
        const treeContainer = $(treeContainerElement);
        const treeJs = treeContainer.find('.entity-reference-tree');
        const fieldEditName = treeContainer.find(".entity-reference-tree-widget-field").val();
        const widgetElement = $("#" + fieldEditName);
        const theme = treeJs.attr("theme");
        const dots = treeJs.attr("dots");

        // Move the inline widget above the description. Claro uses '.form-item__description'. Seven uses '.description'.
        const descriptionElement = treeContainer.closest('.field--type-entity-reference').find('.form-item__description, .description');
        if (descriptionElement.length) {
          treeContainer.insertBefore(descriptionElement);
        }

        // Create selected text container if missing.
        if (!treeContainer.find(".entity-reference-tree-selected-text").length) {
          treeContainer.append('<div class="entity-reference-tree-selected-text"></div>');
        }

        // Avoid ajax callback from running following codes again.
        if (widgetElement.length) {
          const entityType = treeContainer.find(".entity-reference-tree-entity-type").val();
          const bundle = treeContainer.find(".entity-reference-tree-entity-bundle").val();
          const token = settings["entity_tree_token_" + fieldEditName];
          const idIsString = bundle === "*";
          const limit = parseInt(settings["tree_limit_" + fieldEditName]);
          let selectedNodes;

          // Selected nodes.
          if (idIsString) {
            selectedNodes = widgetElement.val().match(/\([a-z 0-9 _]+\)/g);
          } else {
            selectedNodes = widgetElement.val().match(/\((\d+)\)/g);
          }

          // Calculate remaining selected entities.
          let remaining;
          if (limit > 0) {
            remaining = limit + " " + Drupal.t("max");
          } else {
            remaining = Drupal.t("unlimited");
          }

          if (selectedNodes) {
            // Pick up nodes id.
            for (let i = 0; i < selectedNodes.length; i++) {
              // Remove the round brackets.
              if (idIsString) {
                selectedNodes[i] = selectedNodes[i].slice(
                  1,
                  selectedNodes[i].length - 1
                );
              } else {
                selectedNodes[i] = parseInt(
                  selectedNodes[i].slice(1, selectedNodes[i].length - 1),
                  10
                );
              }
            }
          } else {
            selectedNodes = [];
          }

          treeContainer.find(".entity-reference-tree-selected-node").val(widgetElement.val());
          // Initial selected-entities text.
          treeContainer.find(".entity-reference-tree-selected-text").text(
            Drupal.t("Selected") + " (0 " + Drupal.t("of") + " " + remaining + "): " + widgetElement.val()
          );

          // Build the tree.
          treeJs.jstree({
            core: {
              data: {
                url: function () {
                  return Drupal.url(
                    "admin/entity_reference_tree/json/" +
                    entityType +
                    "/" +
                    bundle +
                    "?token=" +
                    token
                  );
                },
                data: function (node) {
                  return {
                    id: node.id,
                    text: node.text,
                    parent: node.parent
                  };
                }
              },
              themes: {
                dots: dots === "1",
                name: theme
              },
              multiple: limit !== 1
            },
            checkbox: {
              three_state: false
            },
            search: {
              show_only_matches: true
            },
            conditionalselect: function (node) {
              // A bundle node can't be selected.
              if (node.data && node.data.isBundle) {
                return false;
              }
              if (limit > 1) {
                return this.get_selected().length < limit || node.state.selected;
              } else {
                // No limit.
                return true;
              }
            },
            plugins: ["search", "changed", "checkbox", "conditionalselect"]
          });

          // Initialize the selected node.
          treeJs.on("ready.jstree", function (e, data) {
            data.instance.select_node(selectedNodes);
          });

          // Selected event.
          treeJs.on("changed.jstree", function (evt, data) {
            // selected node objects.
            const choosedNodes = data.selected;
            const r = [];

            for (let i = 0; i < choosedNodes.length; i++) {
              const node = data.instance.get_node(choosedNodes[i]);
              // node text escaping double quote.
              let nodeText =
                node.text.replace(/"/g, '""') + " (" + node.id + ")";
              // Comma is a special character for autocomplete widget.
              if (
                nodeText.indexOf(",") !== -1 ||
                nodeText.indexOf("'") !== -1
              ) {
                nodeText = '"' + nodeText + '"';
              }
              r.push(nodeText);
            }

            const selectedText = r.join(", ");
            widgetElement.val(selectedText);
            // Selected-entities text.
            treeContainer.find(".entity-reference-tree-selected-text").text(
              Drupal.t("Selected") + " (" + choosedNodes.length + " " + Drupal.t("of") + " " + remaining + "): " + selectedText
            );
          });

          // Search filter box.
          let to = false;
          treeContainer.find(".entity-reference-tree-search").keyup(function () {
            const searchInput = $(this);
            if (to) {
              clearTimeout(to);
            }
            to = setTimeout(function () {
              const v = searchInput.val();
              treeJs.jstree(true).search(v);
            }, 250);
          });
        }
      });
    }
  };
})(jQuery, Drupal, once);
