<?php

namespace Drupal\entity_reference_tree\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'inline_entity_reference_tree_widget' widget.
 *
 * @FieldWidget(
 *   id = "inline_entity_reference_tree_widget",
 *   label = @Translation("Entity reference tree widget - Inline"),
 *   field_types = {
 *     "entity_reference",
 *   },
 *   multiple_values = TRUE
 * )
 */
class InlineEntityReferenceTreeWidget extends EntityReferenceTreeWidget {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {

    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    $theme = $this->getSetting('theme');
    $dots = $this->getSetting('dots');
    $edit_id = $element['target_id']['#id'];
    $entity_type = $element['target_id']['#target_type'];
    $bundles = $element['target_id']['#selection_settings']['target_bundles'];
    $bundles_string = empty($bundles) ? '*' : implode(',', $bundles);

    // Attach libraries with dependencies.
    $element['#attached']['library'][] = 'entity_reference_tree/entity_tree_inline';
    $form['#attached']['library'][] = 'entity_reference_tree/jstree_' . $theme . '_theme';

    // Instance a entity tree builder for this entity type if it exists.
    if (\Drupal::hasService('entity_reference_' . $entity_type . '_tree_builder')) {
      $treeBuilder = \Drupal::service('entity_reference_' . $entity_type . '_tree_builder');
    }
    else {
      $treeBuilder = \Drupal::service('entity_reference_entity_tree_builder');
    }

    $entityTrees = [];

    foreach ($bundles as $bundle_id) {
      $tree = $treeBuilder->loadTree($entity_type, $bundle_id);
      if (!empty($tree)) {
        foreach ($tree as $entity) {
          // Create tree node for each entity.
          // Store them into an array passed to JS.
          // An array in JavaScript is indexed list.
          // JavaScript's array indices are always sequential
          // and start from 0.
          $treeNode = $treeBuilder->createTreeNode($entity);
          // Applies a very permissive XSS/HTML filter for node text.
          $treeNode['text'] = Xss::filterAdmin($treeNode['text']);
          $entityTrees[] = $treeNode;
        }
      }
    }

    // Pass data to js file.
    $element['#attached']['drupalSettings'] = [
      'entity_tree_token_' . $edit_id => \Drupal::csrfToken()->get($bundles_string),
      'tree_limit_' . $edit_id => $this->fieldDefinition->getFieldStorageDefinition()->getCardinality(),
      'widget_type' => 'inline',
    ];

    // All around wrapper.
    $element['tree_container'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => [
        'class' => [
          'inline-entity-reference-tree-wrapper',
        ],
      ],
    ];

    // Search filter box.
    $element['tree_container']['tree_search'] = [
      '#type' => 'textfield',
      '#title' => $this
        ->t('Search'),
      '#size' => 60,
      '#attributes' => [
        'class' => [
          'entity-reference-tree-search',
        ],
      ],
    ];

    // Field element id.
    $element['tree_container']['field_id'] = [
      '#name' => 'field_id',
      '#type' => 'hidden',
      '#weight' => 80,
      '#value' => $edit_id,
      '#attributes' => [
        'class' => [
          'entity-reference-tree-widget-field',
        ],
      ],
    ];

    // Entity type.
    $element['tree_container']['entity_type'] = [
      '#name' => 'entity_type',
      '#type' => 'hidden',
      '#weight' => 80,
      '#value' => $entity_type,
      '#attributes' => [
        'class' => [
          'entity-reference-tree-entity-type',
        ],
      ],
    ];

    // Entity bundle.
    $element['tree_container']['entity_bundle'] = [
      '#name' => 'entity_bundle',
      '#type' => 'hidden',
      '#weight' => 80,
      '#value' => $bundles,
      '#attributes' => [
        'class' => [
          'entity-reference-tree-entity-bundle',
        ],
      ],
    ];

    // JsTree container.
    $element['tree_container']['js_tree'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => [
        'class' => [
          'entity-reference-tree',
        ],
        'theme' => $theme,
        'dots' => $dots,
      ],
    ];

    // Remove dialog link from parent.
    unset($element['dialog_link']);

    // Hide autocomplete element.
    $element['target_id']['#attributes']['class'][] = 'hidden';

    return $element;

  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = [];
    // JsTRee theme.
    $element['theme'] = [
      '#type' => 'radios',
      '#title' => $this->t('JsTree theme'),
      '#default_value' => $this->getSetting('theme'),
      '#required' => TRUE,
      '#options' => [
        'default' => $this->t('Default'),
        'default-dark' => $this->t('Default Dark'),
      ],
    ];
    // Tree dot.
    $element['dots'] = [
      '#type' => 'radios',
      '#title' => $this->t('Dot line'),
      '#default_value' => $this->getSetting('dots'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    // JsTree theme.
    $summary[] = $this->t('JsTree theme: @theme', ['@theme' => $this->getSetting('theme')]);
    $summary[] = $this->t('JsTree dots: @dots', ['@dots' => $this->getSetting('dots') ? $this->t('Yes') : $this->t('No')]);

    return $summary;
  }

}
